﻿using Hotline.CallCenter.Calls;
using Hotline.Orders;
using Hotline.Permissions;
using Hotline.Settings;
using Hotline.Settings.CommonOpinions;
using Hotline.Share.Dtos.Settings;
using Hotline.Share.Enums.FlowEngine;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using MongoDB.Driver;
using SqlSugar;
using System.Reflection.Metadata;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 常用意见接口
    /// </summary>
    public class CommonPController: BaseController
    {
        private readonly ISystemCommonOpinionDomainService _commonOpinionDomainService;
        private readonly ISystemAreaDomainService _systemAreaDomainService;
        private readonly IMapper _mapper;
        private readonly ISessionContext _sessionContext;
        private readonly IRepository<TrCallRecord> _trCallRecordRepository;
        private readonly IOrderRepository _orderRepository;

		public CommonPController(
            ISystemCommonOpinionDomainService commonOpinionDomainService,
            ISystemAreaDomainService systemAreaDomainService,
            ISessionContext sessionContext,
            IRepository<TrCallRecord> trCallRecordRepository,
            IOrderRepository orderRepository,
			IMapper mapper)
        {
            _commonOpinionDomainService = commonOpinionDomainService;
            _systemAreaDomainService= systemAreaDomainService;
            _mapper = mapper;
            _sessionContext = sessionContext;
            _trCallRecordRepository = trCallRecordRepository;
            _orderRepository = orderRepository;
		}

       

        #region 省市区
        /// <summary>
        /// 获取省市区树形
        /// </summary>
        /// <returns></returns>
        [HttpGet("tree-area")]
        [Obsolete("请调用sys/area/tree")]
        public async Task<List<SystemArea>> GetAearTree()
        {
            return await _systemAreaDomainService.GetAreaTree();
        }

        #endregion

        /// <summary>
        /// 首页基础数据
        /// </summary>
        /// <returns></returns>
        [HttpGet("home_data")]
        public async Task<Object> GetHomeData() 
        {
            var tadayTime = DateTime.Now.ToString("yyyy-MM-dd");
			//中心
			if (_sessionContext.OrgIsCenter)
	        {
		        var orderQuery = _orderRepository.Queryable(false, false, false)
			        .Includes(o => o.Workflow, w => w.Steps);
				//今日来电
				var tadayCalls = await _trCallRecordRepository.Queryable()
	                .Where(x => x.CallDirection == Share.Enums.CallCenter.ECallDirection.In && tadayTime.Equals(x.CreatedTime.ToString("yyyy-MM-dd"))).ToListAsync();
				var callNum = tadayCalls.Count();
                var validCallNum = tadayCalls.Where(x => x.Duration > 0 || x.QueueTims > 0 || x.RingTimes > 0).Count();
				//今日接通率
                var answeredNum = tadayCalls.Where(x=>x.Duration > 0).Count();
                var answeredRate = validCallNum > 0 ? (answeredNum / validCallNum * 100) + "%" : "100%";
                //今日受理工单
                var tadayOrders = await orderQuery
	                .Where(o => tadayTime.Equals(o.CreationTime.ToString("yyyy-MM-dd"))).ToListAsync();
				var orderNum = tadayOrders.Count();
				var directlyNum = tadayOrders.Where(o=> o.ProcessType == Share.Enums.Order.EProcessType.Zhiban).Count();
                return new { CallNum = callNum, ValidCallNum = validCallNum, AnsweredNum = answeredNum, AnsweredRate = answeredRate, OrderNum = orderNum, DirectlyNum = directlyNum };
			}
			//部门
			//今日待办 tasksOkNum
			var time = DateTime.Parse(tadayTime);
			var tasksOrder = await _orderRepository.Queryable(false, false, false)
				.Includes(o => o.Workflow, w => w.Steps)
                .Where(o => o.Workflow.Steps.Any(s => s.Status == EWorkflowStepStatus.Handled) && o.Workflow.ExpiredTime > time)
                .GroupBy(o=>new { o.Id, o.Workflow })
                .Select(o => new {
					tasksOkNum= SqlFunc.AggregateSum(SqlFunc.IIF(SqlFunc.JsonListObjectAny(o.Workflow.HandlerUsers, "Key", _sessionContext.RequiredUserId), 1, 0)),
					tasksOkOrgNum = SqlFunc.AggregateSum(SqlFunc.IIF(SqlFunc.JsonListObjectAny(o.Workflow.HandlerOrgs, "Key", _sessionContext.RequiredOrgId), 1, 0)),
				}).FirstAsync();
            var tasksOkNum = tasksOrder.tasksOkNum;
            var tasksOkOrgNum = tasksOrder.tasksOkOrgNum;
            //var tasksOkOrgNum = await _orderRepository.Queryable(false, false, false)
            // .Includes(o => o.Workflow, w => w.Steps)
            //    .Where(o => o.Workflow.Steps.Any(s => s.Status == EWorkflowStepStatus.Handled))
            //    .Where(o => SqlFunc.JsonListObjectAny(o.Workflow.HandlerOrgs, "Key", _sessionContext.RequiredOrgId) && o.Workflow.ExpiredTime > time).CountAsync();
            //今日已办
            var handleOrder = await _orderRepository.Queryable(false, false, false)
                .Includes(o => o.Workflow, w => w.Steps)
                .Where(o => o.Workflow.Steps.Any(s => s.Status != EWorkflowStepStatus.Handled) && o.Workflow.ActualHandleTime.Value.ToString("yyyy-MM-dd") == tadayTime && o.Workflow.ActualHandleTime != null)
				.GroupBy(o => new { o.Id, o.Workflow })
                .Select(o => new {
	                handleNum = SqlFunc.AggregateSum(SqlFunc.IIF( SqlFunc.JsonListObjectAny(o.Workflow.HandlerUsers, "Key", _sessionContext.RequiredUserId), 1, 0)),
	                handleOrgNum = SqlFunc.AggregateSum(SqlFunc.IIF(SqlFunc.JsonListObjectAny(o.Workflow.HandlerOrgs, "Key", _sessionContext.RequiredOrgId), 1, 0)),
                }).FirstAsync();
            var handleNum = handleOrder.handleNum;
            var handleOrgNum = handleOrder.handleOrgNum;
			//var handleNum = await _orderRepository.Queryable(false, false, false)
			//	.Includes(o => o.Workflow, w => w.Steps)
			//             .Where(o => o.Workflow.Steps.Any(s => s.Status != EWorkflowStepStatus.Handled))
			//             .Where(o => o.Workflow.ActualHandleTime != null && SqlFunc.JsonListObjectAny(o.Workflow.HandlerUsers, "Key", _sessionContext.RequiredUserId) && o.Workflow.ActualHandleTime.Value.ToString("yyyy-MM-dd") == tadayTime).CountAsync();
			//         var handleOrgNum = await _orderRepository.Queryable(false, false, false)
			//          .Includes(o => o.Workflow, w => w.Steps)
			//             .Where(o => o.Workflow.Steps.Any(s => s.Status != EWorkflowStepStatus.Handled))
			//             .Where(o => o.Workflow.ActualHandleTime != null && SqlFunc.JsonListObjectAny(o.Workflow.HandlerOrgs, "Key", _sessionContext.RequiredOrgId) && o.Workflow.ActualHandleTime.Value.ToString("yyyy-MM-dd") == tadayTime).CountAsync();
			//部门超期
            var exceedOrder = await _orderRepository.Queryable(false, false, false)
	            .Includes(o => o.Workflow, w => w.Steps)
                .Where(o=> SqlFunc.JsonListObjectAny(o.Workflow.HandlerOrgs, "Key", _sessionContext.RequiredOrgId) && o.ExpiredTime > DateTime.Now)
                .GroupBy(o=>new { o.Id, o.Workflow })
                .Select(o => new {
					exceedTasksOkNum = SqlFunc.AggregateSum(SqlFunc.IIF(o.Workflow.Steps.Any(s => s.Status == EWorkflowStepStatus.Handled), 1, 0)),
					exceedHandleNum = SqlFunc.AggregateSum(SqlFunc.IIF(o.Workflow.Steps.Any(s => s.Status != EWorkflowStepStatus.Handled), 1, 0)),
				}).FirstAsync();
            var exceedTasksOkNum = exceedOrder.exceedTasksOkNum;
            var exceedHandleNum = exceedOrder.exceedHandleNum;
			//var exceedTasksOkNum = await _orderRepository.Queryable(false, false, false)
			//	.Includes(o => o.Workflow, w => w.Steps)
			//             .Where(o => o.Workflow.Steps.Any(s => s.Status == EWorkflowStepStatus.Handled))
			//             .Where(o => SqlFunc.JsonListObjectAny(o.Workflow.HandlerOrgs, "Key", _sessionContext.RequiredOrgId) && o.ExpiredTime > DateTime.Now).CountAsync();
			//         var exceedHandleNum = await _orderRepository.Queryable(false, false, false)
			//          .Includes(o => o.Workflow, w => w.Steps)
			//             .Where(o => o.Workflow.Steps.Any(s => s.Status != EWorkflowStepStatus.Handled))
			//             .Where(o => SqlFunc.JsonListObjectAny(o.Workflow.HandlerOrgs, "Key", _sessionContext.RequiredOrgId) && o.ExpiredTime > DateTime.Now).CountAsync();
			return new { TasksOkNum = tasksOkNum, TasksOkOrgNum = tasksOkOrgNum, HandleNum = handleNum, HandleOrgNum = handleOrgNum, ExceedTasksOkNum = exceedTasksOkNum, ExceedHandleNum = exceedHandleNum };
        }
	}
}
